@@ -12,11 +12,12 @@ npdeptest <- function(data.x = NULL,
   
   ## Trap fatal errors
 
-  if(is.data.frame(data.x)||is.data.frame(data.y)) stop("you must enter two data vectors (and not data frames)")
-  if(is.factor(data.x)||is.factor(data.y)) stop("does not support factors")
-  if(is.null(data.x)||is.null(data.y)) stop("you must enter x and y data vectors")
-  if(ncol(data.frame(data.x)) != 1) stop("data must have one dimension only")
-  if(length(data.x)!=length(data.y)) stop("data vectors must be of equal length")
+  if(is.data.frame(data.x)||is.data.frame(data.y)) stop(" you must enter two data vectors (and not data frames)")
+  if(is.factor(data.x)||is.factor(data.y)) stop(" does not support factors")
+  if(is.null(data.x)||is.null(data.y)) stop(" you must enter x and y data vectors")
+  if(ncol(data.frame(data.x)) != 1) stop(" data must have one dimension only")
+  if(length(data.x)!=length(data.y)) stop(" data vectors must be of equal length")
+  if(boot.num < 9) stop(" number of bootstrap replications must be >= 9")
 
   method <- match.arg(method)
 
@@ -44,25 +45,35 @@ npdeptest <- function(data.x = NULL,
                          bw.x = NULL,
                          bw.y = NULL,
                          bw.joint = NULL,
-                         method=c("summation","integration")) {
+                         method=c("integration","summation")) {
     
-    if(is.null(bw.x)||is.null(bw.y)||is.null(bw.joint)) stop("you must provide numeric bandwidths for f(x), f(y) and f(x,y)")
+    if(is.null(bw.x)||is.null(bw.y)||is.null(bw.joint)) stop(" you must provide numeric bandwidths for f(x), f(y) and f(x,y)")
     
     method <- match.arg(method)
     
     if(method=="summation") {
       
-      ## Summation version:
-      ## \sum_i(1-sqrt(f(x_i)f(y_i)/f(x_i,y_i)))^2
+      ## Summation version: \sum_i(1-sqrt(f(x_i)f(y_i)/f(x_i,y_i)))^2
       ## We could code this by hand per below, however, there is no
       ## performance penalty imposed by sum() so no need.
       
       f.x <- fitted(npudens(tdat=x.dat,bws=bw.x))
       f.y <- fitted(npudens(tdat=y.dat,bws=bw.y))
       f.xy <- fitted(npudens(tdat=cbind(x.dat,y.dat),bws=bw.joint))
-      summand <- ifelse(f.xy > .Machine$double.xmin, f.x*f.y/f.xy, f.x*f.y/(f.xy+.Machine$double.xmin))
+      summand <- f.x*f.y/f.xy
       
-      return(0.5*sum((1-sqrt(summand))**2)/length(x.dat))
+      ## In summation version we divide densities which can lead to
+      ## numerical instability issues not present in the integrand
+      ## version (which uses differences instead). We check for this
+      ## case, remove offending points, and warn. This traps -Inf,
+      ## Inf, and NaN.  
+
+      if(!all(is.finite(summand))) {
+        warning(" non-finite value in summation-based statistic: integration recommended")
+        summand <- summand[is.finite(summand)]
+      }
+
+      return(0.5*mean((1-sqrt(summand))**2))
       
     } else {
       
@@ -84,10 +95,10 @@ npdeptest <- function(data.x = NULL,
         ##      f.y <- fitted(npudens(tdat=y.dat,edat=xy[2],bws=bw.y))
         ##      f.xy <- fitted(npudens(tdat=cbind(x.dat,y.dat),edat=cbind(xy[1],xy[2]),bws=bw.joint))
         
-        f.x <- sum(dnorm((xy[1]-x.dat)/bw.x))/(length(x.dat)*bw.x)
-        f.y <- sum(dnorm((xy[2]-y.dat)/bw.y))/(length(x.dat)*bw.y)
-        f.xy <- sum(dnorm((xy[1]-x.dat)/bw.joint[1])*dnorm((xy[2]-y.dat)/bw.joint[2])
-                    / (length(x.dat)*bw.joint[1]*bw.joint[2]))
+        f.x <- mean(dnorm((xy[1]-x.dat)/bw.x))/bw.x
+        f.y <- mean(dnorm((xy[2]-y.dat)/bw.y))/bw.y
+        f.xy <- mean(dnorm((xy[1]-x.dat)/bw.joint[1])*dnorm((xy[2]-y.dat)/bw.joint[2])
+                    / (bw.joint[1]*bw.joint[2]))
         
         return((sqrt(f.xy)-sqrt(f.x)*sqrt(f.y))**2)
         
@@ -100,10 +111,10 @@ npdeptest <- function(data.x = NULL,
     }
 
   } ## end of Srho.bivar function
-
- console <- newLineConsole()
- console <- printClear(console)
- console <- printPop(console)  
+  
+  console <- newLineConsole()
+  console <- printClear(console)
+  console <- printPop(console)  
   
   ## Compute and save bandwidths (save for bootstrapping if requested)
   
@@ -111,8 +122,8 @@ npdeptest <- function(data.x = NULL,
   bw.data.y <- npudensbw(~data.y)$bw
   bw.joint <- npudensbw(~data.x+data.y)$bw
   
- console <- printClear(console)
- console <- printPush(paste(sep="", "Constructing metric entropy..."), console = console)
+  console <- printClear(console)
+  console <- printPush(paste(sep="", "Constructing metric entropy..."), console = console)
   
   Srho.vec <- Srho.bivar(data.x,data.y,bw.data.x,bw.data.y,bw.joint,method=method)
 
@@ -125,14 +136,14 @@ npdeptest <- function(data.x = NULL,
 
     for(b in 1:boot.num) {
 
-     console <- printClear(console)
-     console <- printPush(paste(sep="", "Bootstrap replication ",
+      console <- printClear(console)
+      console <- printPush(paste(sep="", "Bootstrap replication ",
                                  b, "/", boot.num, "..."), console)
-
+      
       ## Break systematic relationship between x and y (null)
-
+      
       data.x.boot <- sample(data.x,replace=TRUE)
-
+      
       Srho.vec.boot[b] <- Srho.bivar(data.x.boot,data.y,bw.data.x,bw.data.y,bw.joint,method=method)
 
     }
@@ -143,8 +154,8 @@ npdeptest <- function(data.x = NULL,
 
   }
 
- console <- printClear(console)
- console <- printPop(console)
+  console <- printClear(console)
+  console <- printPop(console)
 
   ## Restore seed
 
