@@ -24,11 +24,9 @@ npqcmstest <- function(formula,
                ".\nSee help for further info.",
                sep=""))
 
-  if (tau <=0 | tau >=1)
-    stop("tau must lie in (0,1)")
+  if(tau <=0 | tau >=1) stop("tau must lie in (0,1)")
 
-  if (boot.num < 1)
-    stop("less than one bootstrap replication")
+  if(boot.num < 9) stop("number of bootstrap replications must be >= 9")
 
   ## checking for consistent interface usage
   miss.xy = c(missing(xdat),missing(ydat))
